
# q-expansion of newform 2304.2.l.g, downloaded from the LMFDB on 14 May 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField, ZZ
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K([c/ZZ(den) for c in num]) for num, den in basis_data]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 2304
weight = 2
poly_data = [1, 0, 0, 0, -1, 0, 0, 0, 1]

# The entries in the following list give a basis for the
# coefficient ring in terms of a root of the defining polynomial above.
# Each line consists of the coefficients of the numerator, and a denominator.
basis_data  = [[[1, 0, 0, 0, 0, 0, 0, 0], 1], [[0, 1, 0, 1, 0, 1, 0, 0], 1], [[0, 0, 0, 0, 0, 0, 1, 0], 1], [[0, 0, 0, 0, 0, 0, 0, 2], 1], [[0, 1, 0, -1, 0, -1, 0, 0], 1], [[-1, 0, 2, 0, 2, 0, -1, 0], 1], [[0, 1, 0, 1, 0, -1, 0, 0], 1], [[1, 0, 2, 0, -2, 0, -1, 0], 1]]

hecke_ring_character_values = [[1279, [-1, 0, 0, 0, 0, 0, 0, 0]], [2053, [0, 0, -1, 0, 0, 0, 0, 0]], [1793, [-1, 0, 0, 0, 0, 0, 0, 0]]]
aps_data = [[0, 0, 0, 0, 0, 0, 0, 0], [0, 0, 0, 0, 0, 0, 0, 0], [0, -1, 0, 0, -1, 0, 1, 0], [0, 0, 0, 0, 0, 0, -1, 0], [1, 0, -1, 0, 0, 0, 0, 1], [0, 0, 0, 0, 0, -1, 0, 0], [0, -1, 0, -1, -3, 0, 1, 0], [0, 0, 0, 2, 3, 0, 1, 0], [0, 0, 0, 0, 0, 1, 0, -1], [0, 0, 0, 1, -2, 0, -3, 0], [0, -2, 0, -2, -5, 0, 2, 0], [1, 0, -1, 0, 0, 0, 0, 2], [0, -3, 0, 3, 0, 0, -2, 0], [0, 4, 0, 0, 1, 0, -1, 0], [0, 0, 0, 0, 0, -3, 0, -3], [0, -5, 0, 0, -4, 0, 4, 0], [-2, 0, 2, 0, 0, 0, 0, 4], [-3, 0, -3, 0, 0, 0, 0, 0], [0, 0, 0, -2, -2, 0, 0, 0], [0, 0, 14, 0, 0, 0, 0, 0], [0, 0, -2, 0, 0, -2, 0, 2], [0, -2, 0, -2, 3, 0, 2, 0], [-9, 0, -9, 0, 0, 1, 0, 0], [0, -6, 0, 6, 0, 0, 1, 0], [-8, 0, 0, 0, 0, 3, 0, 3], [0, -5, 0, 0, 0, 0, 0, 0], [0, -6, 0, 6, 0, 0, -3, 0], [10, 0, -10, 0, 0, 0, 0, 2], [-6, 0, -6, 0, 0, 1, 0, 0], [0, 0, 0, 0, -3, 0, 0, 0], [0, -4, 0, -4, -7, 0, 4, 0], [0, 0, 0, 0, 0, 4, 0, 0], [0, -1, 0, 1, 0, 0, -4, 0], [0, 6, 0, 0, 2, 0, -2, 0], [0, -3, 0, 0, -3, 0, 3, 0], [0, 2, 0, -2, 0, 0, 3, 0], [-3, 0, -3, 0, 0, -4, 0, 0], [0, 0, 0, 4, -1, 0, -5, 0], [0, 0, 10, 0, 0, 1, 0, -1], [0, 0, 0, 3, -5, 0, -8, 0], [-6, 0, -6, 0, 0, 0, 0, 0], [6, 0, -6, 0, 0, 0, 0, 3], [10, 0, 0, 0, 0, -1, 0, -1], [2, 0, 0, 0, 0, -6, 0, -6], [0, -5, 0, 0, -10, 0, 10, 0], [0, 4, 0, -4, 0, 0, -7, 0], [0, 0, 0, -2, 8, 0, 10, 0], [0, 0, 0, 0, -9, 0, 0, 0], [13, 0, 13, 0, 0, -1, 0, 0], [2, 0, -2, 0, 0, 0, 0, -3], [0, -2, 0, 2, 0, 0, 11, 0], [-18, 0, 0, 0, 0, 1, 0, 1], [10, 0, 0, 0, 0, 1, 0, 1], [-1, 0, 1, 0, 0, 0, 0, -5], [0, -2, 0, -2, -7, 0, 2, 0], [0, 0, 4, 0, 0, 2, 0, -2], [0, 0, 0, -3, 6, 0, 9, 0], [0, -4, 0, -4, 5, 0, 4, 0], [6, 0, -6, 0, 0, 0, 0, -3], [0, 2, 0, -2, 0, 0, -3, 0], [0, 0, 0, 0, 14, 0, -14, 0], [0, 5, 0, 0, 8, 0, -8, 0], [0, 0, 0, 2, 0, 0, -2, 0], [0, 0, -16, 0, 0, -4, 0, 4], [0, 0, -2, 0, 0, 0, 0, 0], [0, 0, 0, -7, 4, 0, 11, 0], [0, -10, 0, 0, -8, 0, 8, 0], [2, 0, 0, 0, 0, -6, 0, -6], [-13, 0, 13, 0, 0, 0, 0, -7], [-3, 0, -3, 0, 0, 6, 0, 0], [0, 10, 0, 10, 15, 0, -10, 0], [0, 0, 10, 0, 0, -4, 0, 4], [0, 0, 0, 0, 13, 0, 0, 0], [-9, 0, 9, 0, 0, 0, 0, -8], [0, -2, 0, 0, 3, 0, -3, 0], [22, 0, 0, 0, 0, 1, 0, 1], [0, 13, 0, 0, 1, 0, -1, 0], [-1, 0, -1, 0, 0, 10, 0, 0], [0, 1, 0, 1, 15, 0, -1, 0], [0, 0, 4, 0, 0, 3, 0, -3], [9, 0, 9, 0, 0, -1, 0, 0], [16, 0, -16, 0, 0, 0, 0, 1], [-20, 0, 0, 0, 0, 5, 0, 5], [-8, 0, 0, 0, 0, 2, 0, 2], [0, 0, 0, 0, 0, 0, 21, 0], [-11, 0, 11, 0, 0, 0, 0, -9], [0, -3, 0, -3, 5, 0, 3, 0], [0, 0, 4, 0, 0, 5, 0, -5], [0, 0, 0, -5, 11, 0, 16, 0], [0, 8, 0, 8, -3, 0, -8, 0], [-13, 0, -13, 0, 0, 5, 0, 0], [10, 0, 0, 0, 0, 6, 0, 6], [0, 10, 0, -10, 0, 0, 1, 0], [14, 0, -14, 0, 0, 0, 0, -8], [0, 0, 0, -20, -8, 0, 12, 0], [0, 0, -18, 0, 0, 2, 0, -2], [0, 0, 0, 3, 12, 0, 9, 0], [0, 11, 0, -11, 0, 0, -4, 0], [0, -6, 0, 0, -9, 0, 9, 0], [-8, 0, -8, 0, 0, 11, 0, 0], [0, 0, 0, 12, 5, 0, -7, 0], [0, 0, 0, -9, -11, 0, -2, 0], [1, 0, 1, 0, 0, -11, 0, 0], [0, 5, 0, -5, 0, 0, 2, 0], [0, 0, 0, 0, 14, 0, -14, 0], [32, 0, 0, 0, 0, 2, 0, 2], [10, 0, -10, 0, 0, 0, 0, -4], [0, 8, 0, 8, -1, 0, -8, 0], [0, 0, -16, 0, 0, -5, 0, 5], [0, 0, -8, 0, 0, 6, 0, -6], [0, 2, 0, 2, -7, 0, -2, 0], [23, 0, -23, 0, 0, 0, 0, -6], [0, 0, 0, 0, 0, 0, 5, 0], [0, -12, 0, 0, -2, 0, 2, 0], [0, 16, 0, -16, 0, 0, -7, 0], [0, -7, 0, -7, -11, 0, 7, 0], [0, 0, 0, -20, -9, 0, 11, 0], [0, 0, -8, 0, 0, -3, 0, 3], [0, 0, 0, 5, 21, 0, 16, 0], [4, 0, 4, 0, 0, -6, 0, 0], [-1, 0, 1, 0, 0, 0, 0, -12], [12, 0, 0, 0, 0, -6, 0, -6], [0, -7, 0, 0, 13, 0, -13, 0], [-13, 0, 13, 0, 0, 0, 0, 1], [0, 0, 0, 16, 5, 0, -11, 0], [0, 0, 0, -19, -14, 0, 5, 0], [0, 0, 0, 0, 0, 0, 0, -3], [2, 0, 0, 0, 0, -8, 0, -8], [0, 6, 0, -6, 0, 0, 3, 0], [-8, 0, -8, 0, 0, -17, 0, 0], [0, 0, 0, 8, 2, 0, -6, 0], [0, 0, 2, 0, 0, 11, 0, -11], [0, 2, 0, 2, -5, 0, -2, 0], [26, 0, -26, 0, 0, 0, 0, 5], [0, -7, 0, 7, 0, 0, 0, 0], [-18, 0, 0, 0, 0, -8, 0, -8], [0, 9, 0, 0, -12, 0, 12, 0], [0, 0, 0, -6, 9, 0, 15, 0], [0, 0, 0, -3, -11, 0, -8, 0], [0, 3, 0, -3, 0, 0, -14, 0], [0, 6, 0, 0, -7, 0, 7, 0], [0, -9, 0, 0, -2, 0, 2, 0], [0, -10, 0, 10, 0, 0, 5, 0], [-6, 0, 6, 0, 0, 0, 0, 6], [10, 0, 10, 0, 0, 3, 0, 0], [0, 0, 30, 0, 0, -6, 0, 6], [-15, 0, 15, 0, 0, 0, 0, 12], [0, -3, 0, 3, 0, 0, 12, 0], [0, 20, 0, 0, 15, 0, -15, 0], [22, 0, 0, 0, 0, 3, 0, 3], [-15, 0, -15, 0, 0, -2, 0, 0], [0, -6, 0, -6, 3, 0, 6, 0], [0, 0, 0, 8, 17, 0, 9, 0], [0, 0, 40, 0, 0, -2, 0, 2], [0, 8, 0, 0, 5, 0, -5, 0], [-18, 0, 0, 0, 0, -9, 0, -9], [0, 4, 0, -4, 0, 0, -9, 0], [0, -7, 0, -7, -7, 0, 7, 0], [0, 0, -30, 0, 0, 0, 0, 0], [0, 0, 0, -5, -19, 0, -14, 0], [-20, 0, -20, 0, 0, 6, 0, 0], [0, -11, 0, 11, 0, 0, -8, 0], [0, -16, 0, 16, 0, 0, 7, 0], [9, 0, -9, 0, 0, 0, 0, 15], [0, -3, 0, -3, 1, 0, 3, 0], [0, 0, -4, 0, 0, 10, 0, -10], [0, -10, 0, -10, 5, 0, 10, 0], [21, 0, -21, 0, 0, 0, 0, 0], [12, 0, 0, 0, 0, 7, 0, 7], [0, -15, 0, 0, -15, 0, 15, 0], [14, 0, -14, 0, 0, 0, 0, 0], [-11, 0, -11, 0, 0, 2, 0, 0], [0, 0, 6, 0, 0, -9, 0, 9], [0, 0, 4, 0, 0, 2, 0, -2], [0, 4, 0, 4, -7, 0, -4, 0], [0, -10, 0, 10, 0, 0, -15, 0], [0, -4, 0, 0, -22, 0, 22, 0], [0, -1, 0, 0, 29, 0, -29, 0], [0, -10, 0, 10, 0, 0, 17, 0], [-41, 0, -41, 0, 0, 0, 0, 0], [0, -6, 0, -6, -7, 0, 6, 0], [-17, 0, -17, 0, 0, 9, 0, 0], [-26, 0, 26, 0, 0, 0, 0, -5], [0, -14, 0, 14, 0, 0, -17, 0], [26, 0, 0, 0, 0, 3, 0, 3], [0, -11, 0, 0, -1, 0, 1, 0], [-6, 0, -6, 0, 0, 7, 0, 0], [0, 0, 0, -8, 20, 0, 28, 0], [0, 0, 26, 0, 0, -7, 0, 7], [-32, 0, 0, 0, 0, -6, 0, -6], [-40, 0, 0, 0, 0, 6, 0, 6], [-8, 0, 8, 0, 0, 0, 0, 14], [0, 0, 0, 20, 0, 0, -20, 0], [0, 0, 0, -1, -23, 0, -22, 0], [4, 0, 4, 0, 0, -4, 0, 0], [0, -12, 0, 12, 0, 0, -1, 0], [-4, 0, 0, 0, 0, -8, 0, -8], [24, 0, 24, 0, 0, -7, 0, 0], [0, -2, 0, -2, 5, 0, 2, 0], [0, 0, 24, 0, 0, 10, 0, -10], [0, 0, 0, -19, -3, 0, 16, 0], [0, -12, 0, -12, -3, 0, 12, 0], [-12, 0, 12, 0, 0, 0, 0, 17], [14, 0, 0, 0, 0, -4, 0, -4], [-23, 0, 23, 0, 0, 0, 0, 9], [0, 0, 0, -5, 8, 0, 13, 0], [0, -10, 0, -10, 1, 0, 10, 0], [-7, 0, -7, 0, 0, -15, 0, 0], [0, -16, 0, 16, 0, 0, 13, 0], [0, 0, 0, 0, -11, 0, 11, 0], [-2, 0, 0, 0, 0, 2, 0, 2], [0, -21, 0, 0, -9, 0, 9, 0], [0, 6, 0, -6, 0, 0, -1, 0], [24, 0, -24, 0, 0, 0, 0, 6], [0, 0, -6, 0, 0, 6, 0, -6], [0, 0, -16, 0, 0, -10, 0, 10], [0, -2, 0, -2, -41, 0, 2, 0], [0, -21, 0, -21, -31, 0, 21, 0], [0, 0, 18, 0, 0, -3, 0, 3], [0, 0, 0, 11, -11, 0, -22, 0], [24, 0, -24, 0, 0, 0, 0, -1], [0, -14, 0, 14, 0, 0, -25, 0], [0, 11, 0, 11, 7, 0, -11, 0], [0, 0, 0, 0, -7, 0, 0, 0], [5, 0, 5, 0, 0, 7, 0, 0], [19, 0, -19, 0, 0, 0, 0, -10], [0, -20, 0, 20, 0, 0, -15, 0], [-22, 0, 0, 0, 0, 2, 0, 2], [0, 2, 0, -2, 0, 0, -29, 0], [-12, 0, 12, 0, 0, 0, 0, -2], [-8, 0, -8, 0, 0, -25, 0, 0], [0, 0, 0, 12, 5, 0, -7, 0], [0, -4, 0, -4, 7, 0, 4, 0], [0, 10, 0, -10, 0, 0, 1, 0], [0, 0, 0, 0, 25, 0, -25, 0], [34, 0, 0, 0, 0, 4, 0, 4], [-16, 0, 0, 0, 0, 3, 0, 3], [0, 25, 0, 0, 7, 0, -7, 0], [23, 0, -23, 0, 0, 0, 0, 9], [0, 0, -24, 0, 0, -9, 0, 9], [17, 0, 17, 0, 0, -11, 0, 0], [0, 18, 0, 0, 8, 0, -8, 0], [0, -8, 0, 8, 0, 0, 31, 0], [-3, 0, -3, 0, 0, 16, 0, 0], [0, 16, 0, 16, 9, 0, -16, 0], [0, 0, 30, 0, 0, 0, 0, 0], [0, -4, 0, -4, 35, 0, 4, 0], [-8, 0, -8, 0, 0, 12, 0, 0], [0, -34, 0, 0, -14, 0, 14, 0], [24, 0, 0, 0, 0, 8, 0, 8], [10, 0, 10, 0, 0, -15, 0, 0], [0, 0, 0, 0, 29, 0, 0, 0], [0, 0, -60, 0, 0, 1, 0, -1], [0, 0, -24, 0, 0, 2, 0, -2], [0, 0, 0, 7, 26, 0, 19, 0], [26, 0, 26, 0, 0, 2, 0, 0], [-18, 0, 18, 0, 0, 0, 0, 9], [0, 4, 0, 0, -10, 0, 10, 0], [0, -17, 0, 0, 16, 0, -16, 0], [0, 0, 4, 0, 0, 10, 0, -10], [0, 10, 0, 10, 1, 0, -10, 0], [8, 0, 8, 0, 0, 6, 0, 0], [-24, 0, 24, 0, 0, 0, 0, 9], [-25, 0, -25, 0, 0, -4, 0, 0], [0, -13, 0, -13, 17, 0, 13, 0], [0, 0, 0, 2, 20, 0, 18, 0], [0, 0, 0, 3, -15, 0, -18, 0], [0, 4, 0, -4, 0, 0, -37, 0], [0, 22, 0, 0, 21, 0, -21, 0], [0, 31, 0, 0, 1, 0, -1, 0], [21, 0, 21, 0, 0, 12, 0, 0], [0, 0, 0, -8, -22, 0, -14, 0], [0, 0, 64, 0, 0, -2, 0, 2], [0, 18, 0, 18, 41, 0, -18, 0], [-8, 0, 0, 0, 0, 4, 0, 4], [0, -22, 0, 22, 0, 0, -9, 0], [-3, 0, 3, 0, 0, 0, 0, -19], [13, 0, 13, 0, 0, 14, 0, 0], [0, 0, -20, 0, 0, 12, 0, -12], [-8, 0, -8, 0, 0, 28, 0, 0], [-26, 0, 0, 0, 0, 8, 0, 8], [0, 12, 0, -12, 0, 0, 15, 0], [0, 0, 28, 0, 0, -3, 0, 3], [3, 0, -3, 0, 0, 0, 0, -14], [0, -2, 0, 0, -27, 0, 27, 0], [0, 0, 0, 0, 0, 8, 0, 8], [6, 0, 0, 0, 0, -7, 0, -7], [0, 7, 0, 0, 1, 0, -1, 0], [0, 8, 0, -8, 0, 0, 21, 0], [0, -10, 0, -10, -17, 0, 10, 0], [0, 0, 0, -43, -25, 0, 18, 0], [16, 0, 16, 0, 0, -2, 0, 0], [0, 0, 0, 0, 0, 0, 43, 0], [-22, 0, 22, 0, 0, 0, 0, -4], [25, 0, 25, 0, 0, -8, 0, 0], [0, 0, 0, -7, -5, 0, 2, 0], [0, 4, 0, 4, 13, 0, -4, 0], [0, 37, 0, 0, 22, 0, -22, 0], [35, 0, -35, 0, 0, 0, 0, -9], [0, 0, 0, 2, 19, 0, 17, 0], [0, 0, -2, 0, 0, -10, 0, 10], [0, 0, 0, -23, 5, 0, 28, 0], [0, -4, 0, -4, -15, 0, 4, 0]]
